package javax.swing.tree;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.BitSet;
import java.beans.PropertyChangeListener;
import java.util.EventListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Hashtable;
import javax.swing.DefaultListSelectionModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.SwingPropertyChangeSupport;
import java.io.Serializable;

public class DefaultTreeSelectionModel implements Cloneable, Serializable, TreeSelectionModel
{
    public static final String SELECTION_MODE_PROPERTY = "selectionMode";
    protected SwingPropertyChangeSupport changeSupport;
    protected TreePath[] selection;
    protected EventListenerList listenerList;
    protected transient RowMapper rowMapper;
    protected DefaultListSelectionModel listSelectionModel;
    protected int selectionMode;
    protected TreePath leadPath;
    protected int leadIndex;
    protected int leadRow;
    private Hashtable<TreePath, Boolean> uniquePaths;
    private Hashtable<TreePath, Boolean> lastPaths;
    private TreePath[] tempPaths;
    
    public DefaultTreeSelectionModel() {
        this.listenerList = new EventListenerList();
        this.listSelectionModel = new DefaultListSelectionModel();
        this.selectionMode = 4;
        int n = -1;
        this.leadRow = n;
        this.leadIndex = n;
        this.uniquePaths = new Hashtable();
        this.lastPaths = new Hashtable();
        this.tempPaths = new TreePath[1];
    }
    
    @Override
    public void setRowMapper(RowMapper rowMapper) {
        this.rowMapper = rowMapper;
        this.resetRowSelection();
    }
    
    @Override
    public RowMapper getRowMapper() {
        return this.rowMapper;
    }
    
    @Override
    public void setSelectionMode(int selectionMode) {
        int selectionMode2 = this.selectionMode;
        this.selectionMode = selectionMode;
        if (this.selectionMode != 1 && this.selectionMode != 2 && this.selectionMode != 4) {
            this.selectionMode = 4;
        }
        if (selectionMode2 != this.selectionMode && this.changeSupport != null) {
            this.changeSupport.firePropertyChange("selectionMode", Integer.valueOf(selectionMode2), Integer.valueOf(this.selectionMode));
        }
    }
    
    @Override
    public int getSelectionMode() {
        return this.selectionMode;
    }
    
    @Override
    public void setSelectionPath(TreePath treePath) {
        if (treePath == null) {
            this.setSelectionPaths((TreePath[])null);
        }
        else {
            this.setSelectionPaths(new TreePath[] { treePath });
        }
    }
    
    @Override
    public void setSelectionPaths(TreePath[] array) {
        int length;
        if (array == null) {
            length = 0;
        }
        else {
            length = array.length;
        }
        int length2;
        if (this.selection == null) {
            length2 = 0;
        }
        else {
            length2 = this.selection.length;
        }
        if (length + length2 != 0) {
            if (this.selectionMode == 1) {
                if (length > 1) {
                    array = new TreePath[] { null };
                    length = 1;
                }
            }
            else if (this.selectionMode == 2 && length > 0 && !this.arePathsContiguous(array)) {
                array = new TreePath[] { null };
                length = 1;
            }
            TreePath leadPath = this.leadPath;
            Vector o = new Vector(length + length2);
            ArrayList o2 = new ArrayList(length);
            this.lastPaths.clear();
            this.leadPath = null;
            for (TreePath treePath : array) {
                if (treePath != null && this.lastPaths.get(treePath) == null) {
                    this.lastPaths.put(treePath, Boolean.TRUE);
                    if (this.uniquePaths.get(treePath) == null) {
                        o.addElement(new PathPlaceHolder(treePath, true));
                    }
                    o2.add(this.leadPath = treePath);
                }
            }
            TreePath[] selection = (TreePath[])o2.toArray(new TreePath[o2.size()]);
            for (int j = 0; j < length2; ++j) {
                if (this.selection[j] != null && this.lastPaths.get(this.selection[j]) == null) {
                    o.addElement(new PathPlaceHolder(this.selection[j], false));
                }
            }
            this.selection = selection;
            Hashtable<Object, Object> uniquePaths = this.uniquePaths;
            this.uniquePaths = this.lastPaths;
            (this.lastPaths = uniquePaths).clear();
            this.insureUniqueness();
            this.updateLeadIndex();
            this.resetRowSelection();
            if (o.size() > 0) {
                this.notifyPathChange(o, leadPath);
            }
        }
    }
    
    @Override
    public void addSelectionPath(TreePath treePath) {
        if (treePath != null) {
            this.addSelectionPaths(new TreePath[] { treePath });
        }
    }
    
    @Override
    public void addSelectionPaths(TreePath[] array) {
        int n = (array == null) ? 0 : array.length;
        if (n > 0) {
            if (this.selectionMode == 1) {
                this.setSelectionPaths(array);
            }
            else if (this.selectionMode == 2 && !this.canPathsBeAdded(array)) {
                if (this.arePathsContiguous(array)) {
                    this.setSelectionPaths(array);
                }
                else {
                    this.setSelectionPaths(new TreePath[] { array[0] });
                }
            }
            else {
                TreePath leadPath = this.leadPath;
                Object o = null;
                int length;
                if (this.selection == null) {
                    length = 0;
                }
                else {
                    length = this.selection.length;
                }
                this.lastPaths.clear();
                int i = 0;
                int n2 = 0;
                while (i < n) {
                    if (array[i] != null) {
                        if (this.uniquePaths.get(array[i]) == null) {
                            ++n2;
                            if (o == null) {
                                o = new Vector();
                            }
                            o.addElement(new PathPlaceHolder(array[i], true));
                            this.uniquePaths.put(array[i], Boolean.TRUE);
                            this.lastPaths.put(array[i], Boolean.TRUE);
                        }
                        this.leadPath = array[i];
                    }
                    ++i;
                }
                if (this.leadPath == null) {
                    this.leadPath = leadPath;
                }
                if (n2 > 0) {
                    TreePath[] selection = new TreePath[length + n2];
                    if (length > 0) {
                        System.arraycopy(this.selection, 0, selection, 0, length);
                    }
                    if (n2 != array.length) {
                        Enumeration<TreePath> keys = this.lastPaths.keys();
                        while (keys.hasMoreElements()) {
                            selection[length++] = (TreePath)keys.nextElement();
                        }
                    }
                    else {
                        System.arraycopy(array, 0, selection, length, n2);
                    }
                    this.selection = selection;
                    this.insureUniqueness();
                    this.updateLeadIndex();
                    this.resetRowSelection();
                    this.notifyPathChange(o, leadPath);
                }
                else {
                    this.leadPath = leadPath;
                }
                this.lastPaths.clear();
            }
        }
    }
    
    @Override
    public void removeSelectionPath(TreePath treePath) {
        if (treePath != null) {
            this.removeSelectionPaths(new TreePath[] { treePath });
        }
    }
    
    @Override
    public void removeSelectionPaths(TreePath[] array) {
        if (array != null && this.selection != null && array.length > 0) {
            if (!this.canPathsBeRemoved(array)) {
                this.clearSelection();
            }
            else {
                Object o = null;
                for (int i = array.length - 1; i >= 0; --i) {
                    if (array[i] != null && this.uniquePaths.get(array[i]) != null) {
                        if (o == null) {
                            o = new Vector(array.length);
                        }
                        this.uniquePaths.remove(array[i]);
                        o.addElement(new PathPlaceHolder(array[i], false));
                    }
                }
                if (o != null) {
                    int size = o.size();
                    TreePath leadPath = this.leadPath;
                    if (size == this.selection.length) {
                        this.selection = null;
                    }
                    else {
                        Enumeration<TreePath> keys = this.uniquePaths.keys();
                        int n = 0;
                        this.selection = new TreePath[this.selection.length - size];
                        while (keys.hasMoreElements()) {
                            this.selection[n++] = (TreePath)keys.nextElement();
                        }
                    }
                    if (this.leadPath != null && this.uniquePaths.get(this.leadPath) == null) {
                        if (this.selection != null) {
                            this.leadPath = this.selection[this.selection.length - 1];
                        }
                        else {
                            this.leadPath = null;
                        }
                    }
                    else if (this.selection != null) {
                        this.leadPath = this.selection[this.selection.length - 1];
                    }
                    else {
                        this.leadPath = null;
                    }
                    this.updateLeadIndex();
                    this.resetRowSelection();
                    this.notifyPathChange(o, leadPath);
                }
            }
        }
    }
    
    @Override
    public TreePath getSelectionPath() {
        if (this.selection != null && this.selection.length > 0) {
            return this.selection[0];
        }
        return null;
    }
    
    @Override
    public TreePath[] getSelectionPaths() {
        if (this.selection != null) {
            int length = this.selection.length;
            TreePath[] array = new TreePath[length];
            System.arraycopy(this.selection, 0, array, 0, length);
            return array;
        }
        return new TreePath[0];
    }
    
    @Override
    public int getSelectionCount() {
        return (this.selection == null) ? 0 : this.selection.length;
    }
    
    @Override
    public boolean isPathSelected(TreePath key) {
        return key != null && this.uniquePaths.get(key) != null;
    }
    
    @Override
    public boolean isSelectionEmpty() {
        return this.selection == null || this.selection.length == 0;
    }
    
    @Override
    public void clearSelection() {
        if (this.selection != null && this.selection.length > 0) {
            int length = this.selection.length;
            boolean[] areNew = new boolean[length];
            for (int i = 0; i < length; ++i) {
                areNew[i] = false;
            }
            TreeSelectionEvent treeSelectionEvent = new TreeSelectionEvent(this, this.selection, areNew, this.leadPath, null);
            this.leadPath = null;
            int n = -1;
            this.leadRow = n;
            this.leadIndex = n;
            this.uniquePaths.clear();
            this.selection = null;
            this.resetRowSelection();
            this.fireValueChanged(treeSelectionEvent);
        }
    }
    
    @Override
    public void addTreeSelectionListener(TreeSelectionListener l) {
        this.listenerList.add(TreeSelectionListener.class, l);
    }
    
    @Override
    public void removeTreeSelectionListener(TreeSelectionListener l) {
        this.listenerList.remove(TreeSelectionListener.class, l);
    }
    
    public TreeSelectionListener[] getTreeSelectionListeners() {
        return (TreeSelectionListener[])this.listenerList.getListeners(TreeSelectionListener.class);
    }
    
    protected void fireValueChanged(TreeSelectionEvent treeSelectionEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeSelectionListener.class) {
                ((TreeSelectionListener)listenerList[i + 1]).valueChanged(treeSelectionEvent);
            }
        }
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
    
    @Override
    public int[] getSelectionRows() {
        if (this.rowMapper != null && this.selection != null && this.selection.length > 0) {
            int[] rowsForPaths = this.rowMapper.getRowsForPaths(this.selection);
            if (rowsForPaths != null) {
                int n = 0;
                for (int i = rowsForPaths.length - 1; i >= 0; --i) {
                    if (rowsForPaths[i] == -1) {
                        ++n;
                    }
                }
                if (n > 0) {
                    if (n == rowsForPaths.length) {
                        rowsForPaths = null;
                    }
                    else {
                        int[] array = new int[rowsForPaths.length - n];
                        int j = rowsForPaths.length - 1;
                        int n2 = 0;
                        while (j >= 0) {
                            if (rowsForPaths[j] != -1) {
                                array[n2++] = rowsForPaths[j];
                            }
                            --j;
                        }
                        rowsForPaths = array;
                    }
                }
            }
            return rowsForPaths;
        }
        else {
            return new int[0];
        }
    }
    
    @Override
    public int getMinSelectionRow() {
        return this.listSelectionModel.getMinSelectionIndex();
    }
    
    @Override
    public int getMaxSelectionRow() {
        return this.listSelectionModel.getMaxSelectionIndex();
    }
    
    @Override
    public boolean isRowSelected(int index) {
        return this.listSelectionModel.isSelectedIndex(index);
    }
    
    @Override
    public void resetRowSelection() {
        this.listSelectionModel.clearSelection();
        if (this.selection != null && this.rowMapper != null) {
            int[] rowsForPaths = this.rowMapper.getRowsForPaths(this.selection);
            for (int i = 0; i < this.selection.length; ++i) {
                int n = rowsForPaths[i];
                if (n != -1) {
                    this.listSelectionModel.addSelectionInterval(n, n);
                }
            }
            if (this.leadIndex != -1 && rowsForPaths != null) {
                this.leadRow = rowsForPaths[this.leadIndex];
            }
            else if (this.leadPath != null) {
                this.tempPaths[0] = this.leadPath;
                int[] rowsForPaths2 = this.rowMapper.getRowsForPaths(this.tempPaths);
                this.leadRow = ((rowsForPaths2 != null) ? rowsForPaths2[0] : -1);
            }
            else {
                this.leadRow = -1;
            }
            this.insureRowContinuity();
        }
        else {
            this.leadRow = -1;
        }
    }
    
    @Override
    public int getLeadSelectionRow() {
        return this.leadRow;
    }
    
    @Override
    public TreePath getLeadSelectionPath() {
        return this.leadPath;
    }
    
    @Override
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            this.changeSupport = new SwingPropertyChangeSupport(this);
        }
        this.changeSupport.addPropertyChangeListener(listener);
    }
    
    @Override
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            return;
        }
        this.changeSupport.removePropertyChangeListener(listener);
        return;
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners() {
        if (this.changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return this.changeSupport.getPropertyChangeListeners();
    }
    
    protected void insureRowContinuity() {
        if (this.selectionMode == 2 && this.selection != null && this.rowMapper != null) {
            DefaultListSelectionModel listSelectionModel = this.listSelectionModel;
            int i = listSelectionModel.getMinSelectionIndex();
            if (i != -1) {
                for (; i <= listSelectionModel.getMaxSelectionIndex(); ++i) {
                    if (!listSelectionModel.isSelectedIndex(i)) {
                        if (i == i) {
                            this.clearSelection();
                        }
                        else {
                            TreePath[] selectionPaths = new TreePath[i - i];
                            int[] rowsForPaths = this.rowMapper.getRowsForPaths(this.selection);
                            for (int j = 0; j < rowsForPaths.length; ++j) {
                                if (rowsForPaths[j] < i) {
                                    selectionPaths[rowsForPaths[j] - i] = this.selection[j];
                                }
                            }
                            this.setSelectionPaths(selectionPaths);
                            break;
                        }
                    }
                }
            }
        }
        else if (this.selectionMode == 1 && this.selection != null && this.selection.length > 1) {
            this.setSelectionPath(this.selection[0]);
        }
    }
    
    protected boolean arePathsContiguous(TreePath[] array) {
        if (this.rowMapper == null || array.length < 2) {
            return true;
        }
        BitSet set = new BitSet(32);
        int length = array.length;
        int n = 0;
        TreePath[] array2 = { array[0] };
        int i = this.rowMapper.getRowsForPaths(array2)[0];
        for (int j = 0; j < length; ++j) {
            if (array[j] != null) {
                array2[0] = array[j];
                int[] rowsForPaths = this.rowMapper.getRowsForPaths(array2);
                if (rowsForPaths == null) {
                    return false;
                }
                int n2 = rowsForPaths[0];
                if (n2 == -1 || n2 < i - length || n2 > i + length) {
                    return false;
                }
                if (n2 < i) {
                    i = n2;
                }
                if (!set.get(n2)) {
                    set.set(n2);
                    ++n;
                }
            }
        }
        int n3 = n + i;
        while (i < n3) {
            if (!set.get(i)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    protected boolean canPathsBeAdded(TreePath[] array) {
        if (array == null || array.length == 0 || this.rowMapper == null || this.selection == null || this.selectionMode == 4) {
            return true;
        }
        BitSet set = new BitSet();
        DefaultListSelectionModel listSelectionModel = this.listSelectionModel;
        int i = listSelectionModel.getMinSelectionIndex();
        int b = listSelectionModel.getMaxSelectionIndex();
        TreePath[] array2 = { null };
        if (i != -1) {
            for (; i <= b; ++i) {
                if (listSelectionModel.isSelectedIndex(i)) {
                    set.set(i);
                }
            }
        }
        else {
            array2[0] = array[0];
            b = (i = this.rowMapper.getRowsForPaths(array2)[0]);
        }
        for (int j = array.length - 1; j >= 0; --j) {
            if (array[j] != null) {
                array2[0] = array[j];
                int[] rowsForPaths = this.rowMapper.getRowsForPaths(array2);
                if (rowsForPaths == null) {
                    return false;
                }
                int bitIndex = rowsForPaths[0];
                i = Math.min(bitIndex, i);
                b = Math.max(bitIndex, b);
                if (bitIndex == -1) {
                    return false;
                }
                set.set(bitIndex);
            }
        }
        while (i <= b) {
            if (!set.get(i)) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    protected boolean canPathsBeRemoved(TreePath[] array) {
        if (this.rowMapper == null || this.selection == null || this.selectionMode == 4) {
            return true;
        }
        BitSet set = new BitSet();
        int length = array.length;
        int min = -1;
        int n = 0;
        TreePath[] array2 = { null };
        this.lastPaths.clear();
        for (int i = 0; i < length; ++i) {
            if (array[i] != null) {
                this.lastPaths.put(array[i], Boolean.TRUE);
            }
        }
        for (int j = this.selection.length - 1; j >= 0; --j) {
            if (this.lastPaths.get(this.selection[j]) == null) {
                array2[0] = this.selection[j];
                int[] rowsForPaths = this.rowMapper.getRowsForPaths(array2);
                if (rowsForPaths != null && rowsForPaths[0] != -1 && !set.get(rowsForPaths[0])) {
                    ++n;
                    if (min == -1) {
                        min = rowsForPaths[0];
                    }
                    else {
                        min = Math.min(min, rowsForPaths[0]);
                    }
                    set.set(rowsForPaths[0]);
                }
            }
        }
        this.lastPaths.clear();
        if (n > 1) {
            int k = min + n - 1;
            while (k >= min) {
                if (!set.get(k)) {
                    return false;
                }
                --k;
            }
        }
        return true;
    }
    
    @Deprecated
    protected void notifyPathChange(Vector<?> vector, TreePath oldLeadSelectionPath) {
        int size = vector.size();
        boolean[] areNew = new boolean[size];
        TreePath[] paths = new TreePath[size];
        for (int i = 0; i < size; ++i) {
            PathPlaceHolder pathPlaceHolder = (PathPlaceHolder)vector.elementAt(i);
            areNew[i] = pathPlaceHolder.isNew;
            paths[i] = pathPlaceHolder.path;
        }
        this.fireValueChanged(new TreeSelectionEvent(this, paths, areNew, oldLeadSelectionPath, this.leadPath));
    }
    
    protected void updateLeadIndex() {
        if (this.leadPath != null) {
            if (this.selection == null) {
                this.leadPath = null;
                int n2 = -1;
                this.leadRow = n2;
                this.leadIndex = n2;
            }
            else {
                int n = -1;
                this.leadIndex = n;
                this.leadRow = n;
                int i = this.selection.length - 1;
                while (i >= 0) {
                    if (this.selection[i] == this.leadPath) {
                        this.leadIndex = i;
                        break;
                    }
                    --i;
                }
            }
        }
        else {
            this.leadIndex = -1;
        }
    }
    
    protected void insureUniqueness() {
    }
    
    @Override
    public String toString() {
        int selectionCount = this.getSelectionCount();
        StringBuffer sb = new StringBuffer();
        int[] rowsForPaths;
        if (this.rowMapper != null) {
            rowsForPaths = this.rowMapper.getRowsForPaths(this.selection);
        }
        else {
            rowsForPaths = null;
        }
        sb.append(this.getClass().getName() + " " + this.hashCode() + " [ ");
        for (int i = 0; i < selectionCount; ++i) {
            if (rowsForPaths != null) {
                sb.append(this.selection[i].toString() + "@" + Integer.toString(rowsForPaths[i]) + " ");
            }
            else {
                sb.append(this.selection[i].toString() + " ");
            }
        }
        sb.append("]");
        return sb.toString();
    }
    
    public Object clone() throws CloneNotSupportedException {
        DefaultTreeSelectionModel defaultTreeSelectionModel = (DefaultTreeSelectionModel)super.clone();
        defaultTreeSelectionModel.changeSupport = null;
        if (this.selection != null) {
            int length = this.selection.length;
            defaultTreeSelectionModel.selection = new TreePath[length];
            System.arraycopy(this.selection, 0, defaultTreeSelectionModel.selection, 0, length);
        }
        defaultTreeSelectionModel.listenerList = new EventListenerList();
        defaultTreeSelectionModel.listSelectionModel = (DefaultListSelectionModel)this.listSelectionModel.clone();
        defaultTreeSelectionModel.uniquePaths = new Hashtable();
        defaultTreeSelectionModel.lastPaths = new Hashtable();
        defaultTreeSelectionModel.tempPaths = new TreePath[1];
        return defaultTreeSelectionModel;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        Object[] obj;
        if (this.rowMapper != null && this.rowMapper instanceof Serializable) {
            obj = new Object[] { "rowMapper", this.rowMapper };
        }
        else {
            obj = new Object[0];
        }
        objectOutputStream.writeObject(obj);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        Object[] array = (Object[])(Object[])objectInputStream.readObject();
        if (array.length > 0 && array[0].equals("rowMapper")) {
            this.rowMapper = (RowMapper)array[1];
        }
    }
}
